@@ -1,5 +1,6 @@
 from __future__ import absolute_import, print_function, division
 
+from datetime import date
 import unittest
 
 from pony.orm.core import *
@@ -163,7 +164,7 @@ class TestAttribute(unittest.TestCase):
             attr2 = Required('Entity1')
         db.generate_mapping()
 
-    @raises_exception(ERDiagramError, 'Ambiguous reverse attribute for Entity1.a')
+    @raises_exception(ERDiagramError, "Ambiguous reverse attribute for Entity1.a. Use the 'reverse' parameter for pointing to right attribute")
     def test_attribute19(self):
         db = Database('sqlite', ':memory:')
         class Entity1(db.Entity):
@@ -176,7 +177,7 @@ class TestAttribute(unittest.TestCase):
             d = Set(Entity1)
         db.generate_mapping()
 
-    @raises_exception(ERDiagramError, 'Ambiguous reverse attribute for Entity1.c')
+    @raises_exception(ERDiagramError, "Ambiguous reverse attribute for Entity1.c. Use the 'reverse' parameter for pointing to right attribute")
     def test_attribute20(self):
         db = Database('sqlite', ':memory:')
         class Entity1(db.Entity):
@@ -383,6 +384,26 @@ class TestAttribute(unittest.TestCase):
             attr1 = Set('Entity1', reverse='attr1', table=[1, 'T1'])
         db.generate_mapping(create_tables=True)
 
+    def test_columns_21(self):
+        db = Database('sqlite', ':memory:')
+        class Stat(db.Entity):
+            webinarshow = Optional('WebinarShow')
+        class WebinarShow(db.Entity):
+            stats = Required('Stat')
+        db.generate_mapping(create_tables=True)
+        self.assertEqual(Stat.webinarshow.column, None)
+        self.assertEqual(WebinarShow.stats.column, 'stats')
+        
+    def test_columns_22(self):
+        db = Database('sqlite', ':memory:')
+        class ZStat(db.Entity):
+            webinarshow = Optional('WebinarShow')
+        class WebinarShow(db.Entity):
+            stats = Required('ZStat')
+        db.generate_mapping(create_tables=True)
+        self.assertEqual(ZStat.webinarshow.column, None)
+        self.assertEqual(WebinarShow.stats.column, 'stats')
+
     def test_nullable1(self):
         db = Database('sqlite', ':memory:')
         class Entity1(db.Entity):
@@ -432,5 +453,79 @@ class TestAttribute(unittest.TestCase):
             b = Set(lambda: db1.Entity1)
         db1.generate_mapping(create_tables=True)
 
+    @raises_exception(ValueError, 'Check for attribute Entity1.a failed. Value: 1')
+    def test_py_check_1(self):
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Required(int, py_check=lambda val: val > 5 and val < 10)
+        db.generate_mapping(create_tables=True)
+        with db_session:
+            obj = Entity1(a=1)
+
+    def test_py_check_2(self):
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Required(int, py_check=lambda val: val > 5 and val < 10)
+        db.generate_mapping(create_tables=True)
+        with db_session:
+            obj = Entity1(a=7)
+
+    def test_py_check_3(self):
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Optional(date, py_check=lambda val: val.year >= 2000)
+        db.generate_mapping(create_tables=True)
+        with db_session:
+            obj = Entity1(a=None)
+
+    @raises_exception(ValueError, 'Check for attribute Entity1.a failed. Value: datetime.date(1999, 1, 1)')
+    def test_py_check_4(self):
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Optional(date, py_check=lambda val: val.year >= 2000)
+        db.generate_mapping(create_tables=True)
+        with db_session:
+            obj = Entity1(a=date(1999, 1, 1))
+
+    def test_py_check_5(self):
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Optional(date, py_check=lambda val: val.year >= 2000)
+        db.generate_mapping(create_tables=True)
+        with db_session:
+            obj = Entity1(a=date(2010, 1, 1))
+
+    @raises_exception(ValueError, 'Should be positive number')
+    def test_py_check_6(self):
+        def positive_number(val):
+            if val <= 0: raise ValueError('Should be positive number')
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Optional(int, py_check=positive_number)
+        db.generate_mapping(create_tables=True)
+        with db_session:
+            obj = Entity1(a=-1)
+
+    def test_py_check_7(self):
+        def positive_number(val):
+            if val <= 0: raise ValueError('Should be positive number')
+            return True
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Optional(int, py_check=positive_number)
+        db.generate_mapping(create_tables=True)
+        with db_session:
+            obj = Entity1(a=1)
+
+    @raises_exception(NotImplementedError, "'py_check' parameter is not supported for collection attributes")
+    def test_py_check_8(self):
+        db = Database('sqlite', ':memory:')
+        class Entity1(db.Entity):
+            a = Required('Entity2')
+        class Entity2(db.Entity):
+            a = Set('Entity1', py_check=lambda val: True)
+        db.generate_mapping(create_tables=True)
+
+
 if __name__ == '__main__':
     unittest.main()
